﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Maps.MapControl;
using BingMapTest.DataService;

/*
Example by Kory Becker
http://www.primaryobjects.com

This class is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
*/

namespace BingMapTest
{
    public partial class MainPage : UserControl
    {
        private DataService.DataServiceClient _client = new DataServiceClient();

        protected enum EditState
        {
            On,
            Off
        };
        protected EditState editState = EditState.Off;

        private Pushpin _editPin = null;
        protected Pushpin editPin
        {
            get
            {
                return _editPin;
            }
            set
            {
                _editPin = value;

                // Update the editable state. If an editPin is available, we are in the editing state.
                editState = editPin == null ? EditState.Off : EditState.On;
            }
        }


        public MainPage()
        {
            InitializeComponent();

            // Initialize the map.
            ctlMap.Center = new Location(40.13, -74.46);
            ctlMap.ZoomLevel = 8;

            // Add a click event handler for adding new pushpins.
            ctlMap.MouseClick += new EventHandler<MapMouseEventArgs>(ctlMap_MouseClick);

            // Load the pushpins from the database.
            LoadPushPins();
        }

        private void LoadPushPins()
        {
            _client.GetLocationsCompleted += new EventHandler<GetLocationsCompletedEventArgs>(_client_GetLocationsCompleted);
            _client.GetLocationsAsync();
        }

        void _client_GetLocationsCompleted(object sender, GetLocationsCompletedEventArgs e)
        {
            Pushpin pushpin = null;

            foreach (LocationViewModel locationViewModel in e.Result)
            {
                // Create a pushpin.
                pushpin = CreatePushpin(locationViewModel.LocationId, locationViewModel.Latitude, locationViewModel.Longitude, locationViewModel.Title, locationViewModel.Description);

                // Add the pushpin to the map.
                ctlMap.Children.Add(pushpin);
            }
        }

        void ctlMap_MouseClick(object sender, MapMouseEventArgs e)
        {
            // The user clicked the map. If we're not in the edit state, add a new pushpin.
            if (editState == EditState.Off)
            {
                // Convert the point to a Location.
                Location location = ctlMap.Mode.ViewportPointToLocation(e.ViewportPoint);

                // Create a pushpin.
                Pushpin pin = CreatePushpin(0, location.Latitude, location.Longitude, location.Latitude + ", " + location.Longitude, "Click the Pushpin to Edit");

                // Add the pushpin to the map.
                ctlMap.Children.Add(pin);
            }
        }

        void pin_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            // The user clicked a pushpin. If we're not in the edit state, show the textbox to edit this pushpin's text.
            if (editState == EditState.Off)
            {
                // Store a copy of the pushpin we're editing.
                editPin = (Pushpin)sender;

                ShowInfoBox();
            }
        }

        void pin_MouseEnter(object sender, MouseEventArgs e)
        {
            // The user is hovering over a pushpin. If we're not in the edit state, show the infobox.
            if (editState == EditState.Off)
            {
                Pushpin pin = (Pushpin)sender;

                // Use the pushpin's Name property for the title and the Tag property for the description.
                InfoboxTitle.Text = pin.Name;
                InfoboxDescription.Text = pin.Tag.ToString();

                // Show the infobox.
                Infobox.Visibility = Visibility.Visible;
                MapLayer.SetPosition(Infobox, pin.Location);
                MapLayer.SetPositionOrigin(Infobox, PositionOrigin.BottomCenter);
                MapLayer.SetPositionOffset(Infobox, new Point(0, -50));
            }
        }

        void pin_MouseLeave(object sender, MouseEventArgs e)
        {
            // The user has moved the mouse away from the pushpin. If we're not in the edit state, hide the infobox.
            if (editState == EditState.Off)
            {
                Infobox.Visibility = Visibility.Collapsed;
            }
        }

        private void btnClose_Click(object sender, RoutedEventArgs e)
        {
            // The user clicked the 'X' close button from editing a pushpin's text.
            HideInfoBox();

            // Save text changes in the pushpin.
            editPin.Tag = InfoboxDescriptionEdit.Text;

            // Save to the database.
            LocationViewModel locationViewModel = new LocationViewModel();
            locationViewModel.LocationId = editPin.TabIndex; // ID
            locationViewModel.Latitude = editPin.Location.Latitude;
            locationViewModel.Longitude = editPin.Location.Longitude;
            locationViewModel.Title = editPin.Name;
            locationViewModel.Description = editPin.Tag.ToString();
            locationViewModel.CreateDate = DateTime.Now;

            _client.SaveLocationCompleted += new EventHandler<SaveLocationCompletedEventArgs>(_client_SaveLocationCompleted);
            _client.SaveLocationAsync(locationViewModel);
        }

        void _client_SaveLocationCompleted(object sender, SaveLocationCompletedEventArgs e)
        {
            if (editState == EditState.On)
            {
                // Update ID for this pushpin.
                editPin.TabIndex = e.Result;

                // Clear out the edit pushpin in memory and reset state.
                editPin = null;
            }
        }

        void btnClear_Click(object sender, RoutedEventArgs e)
        {
            if (editState == EditState.Off)
            {
                // The user clicked the Clear button, erase all pushpins on the map.
                List<UIElement> elementsToRemove = new List<UIElement>();

                // Gather a list of all pushpins on the map.
                foreach (UIElement element in ctlMap.Children)
                {
                    if (element.GetType() == typeof(Pushpin))
                    {
                        Pushpin pin = (Pushpin)element;
                        if (pin != null)
                        {
                            elementsToRemove.Add(element);
                        }
                    }
                }

                // Remove the pushpins from the map.
                foreach (UIElement element in elementsToRemove)
                {
                    ctlMap.Children.Remove(element);
                }
            }
        }

        private void btnDelete_Click(object sender, RoutedEventArgs e)
        {
            if (editState == EditState.On)
            {
                // Delete the pushpin.
                LocationViewModel locationViewModel = new LocationViewModel();
                locationViewModel.LocationId = editPin.TabIndex; // ID
                locationViewModel.Latitude = editPin.Location.Latitude;
                locationViewModel.Longitude = editPin.Location.Longitude;
                locationViewModel.Title = editPin.Name;
                locationViewModel.Description = editPin.Tag.ToString();

                _client.DeleteLocationAsync(locationViewModel);

                HideInfoBox();

                ctlMap.Children.Remove(editPin);
                editPin = null;
            }
        }

        #region Helper Functions

        private void HideInfoBox()
        {
            // Hide editable text box, close button, and delete button.
            InfoboxDescription.Visibility = Visibility.Visible;
            InfoboxDescriptionEdit.Visibility = Visibility.Collapsed;
            btnClose.Visibility = Visibility.Collapsed;
            btnDelete.Visibility = Visibility.Collapsed;
            Infobox.Visibility = Visibility.Collapsed;
        }

        private void ShowInfoBox()
        {
            // Show editable text box, close button, and delete button.
            InfoboxDescription.Visibility = Visibility.Collapsed;
            InfoboxDescriptionEdit.Visibility = Visibility.Visible;
            InfoboxDescriptionEdit.Text = editPin.Tag.ToString();
            btnClose.Visibility = Visibility.Visible;
            btnDelete.Visibility = Visibility.Visible;
        }

        private Pushpin CreatePushpin(int id, double latitude, double longitude, string title, string description)
        {
            Pushpin pushpin = new Pushpin();
            pushpin.TabIndex = id; // ID
            pushpin.Location = new Location(latitude, longitude);
            pushpin.Name = title;
            pushpin.Tag = description;

            // Add pushpin event handlers.
            pushpin.MouseEnter += new MouseEventHandler(pin_MouseEnter);
            pushpin.MouseLeave += new MouseEventHandler(pin_MouseLeave);
            pushpin.MouseLeftButtonUp += new MouseButtonEventHandler(pin_MouseLeftButtonUp);

            return pushpin;
        }

        #endregion
    }
}
